"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = __importStar(require("fs"));
var path_1 = __importDefault(require("path"));
var handlebars_1 = __importDefault(require("handlebars"));
var console_1 = require("../services/console");
var dashify_1 = __importDefault(require("dashify"));
var constants_1 = require("../services/constants");
var colors_1 = __importDefault(require("colors"));
var sdk_dependency_versions_1 = require("../sdk-dependency-versions");
var lodash_1 = require("lodash");
function init(program, projectPath, options) {
    var verbosity = program.opts().verbose;
    var logger = new console_1.Logger(verbosity);
    logger.verbose('Run init');
    var augOps = getAugmentedInitOptions(options, projectPath);
    var files = getFilesToCreate(projectPath, augOps);
    checkFolderEmptyness(program, Object.values(files));
    logger.ok('Creating Mamoru project files');
    createFile(logger, augOps, constants_1.TEMPLATES.PACKAGE_JSON, files.PACKAGE_JSON);
    createFile(logger, augOps, constants_1.TEMPLATES.MANIFEST, files.MANIFEST);
    createFile(logger, augOps, constants_1.TEMPLATES.README, files.README);
    createFile(logger, augOps, constants_1.TEMPLATES.GITIGNORE, files.GITIGNORE);
    if (options.type === 'sql') {
        createFile(logger, augOps, constants_1.TEMPLATES.QUERIES, files.QUERIES);
    }
    if (options.type === 'wasm') {
        createFolder(logger, projectPath, 'src');
        createFolder(logger, projectPath, 'test');
        createFile(logger, augOps, constants_1.TEMPLATES.WASM_INDEX, files.WASM_INDEX);
        createFile(logger, augOps, constants_1.TEMPLATES.WASM_TEST, files.WASM_TEST);
    }
    logger.ok('Mamoru project files created!');
    if (options.type === 'wasm') {
        logger.log("\n    \u2139\uFE0F To start working on your project, run:\n        \n        ".concat(colors_1.default.grey("cd ".concat(projectPath)), "\n        ").concat(colors_1.default.grey("npm install"), "\n    \n    \u2139\uFE0F To start the project locally, run:\n        \n        ").concat(colors_1.default.grey("npm run start"), "\n    \n    \u2139\uFE0F To know more about how to create your own daemons, visit:\n        \n        ").concat(colors_1.default.underline.blue('https://mamoru-foundation.github.io/guides/using-the-mamoru-cli.html'), "\n    "));
    }
    if (options.type === 'sql') {
        logger.log("\n    \u2139\uFE0F To start working on your project, run:\n        \n        ".concat(colors_1.default.grey("cd ".concat(projectPath)), "\n    \n    \u2139\uFE0F To know more about how to create your own daemons, visit:\n        \n        ").concat(colors_1.default.underline.blue('https://mamoru-foundation.github.io/guides/using-the-mamoru-cli.html'), "\n    "));
    }
}
function checkFolderEmptyness(program, paths) {
    paths.forEach(function (p) {
        if (fs.existsSync(p)) {
            var fileName = path_1.default.basename(p);
            program.error("Directory  already contains  a file named \"".concat(fileName, "\", stopping..."));
        }
    });
}
function getAugmentedInitOptions(options, projectPath) {
    var name;
    if (!options.name && !(0, dashify_1.default)(projectPath)) {
        name = 'Default name';
    }
    else {
        name = options.name || (0, lodash_1.deburr)(path_1.default.basename(projectPath));
    }
    return __assign(__assign({}, options), { name: name, jsonTags: JSON.stringify(options.tags.split(',')), kebabName: (0, dashify_1.default)(name), defaultQuery: getDefaultQuery(options.chain), mamoruSdkAsVersion: sdk_dependency_versions_1.sdkVersions.sdk, mamoruCustomSdkPackageName: getCustomSdkPackage(options.chain), mamoruCustomSdkPackageVersion: getCustomSdkPackageVersion(options.chain), customSdkCtxName: getCustomSdkCtxName(options.chain) });
}
function getDefaultQuery(type) {
    switch (type) {
        case 'SUI_MAINNET':
        case 'SUI_TESTNET':
            return "SELECT 1 FROM transactions t WHERE starts_with(t.digest, '0x1_this_is_an_example_query')";
        case 'BSC_TESTNET':
        case 'BSC_MAINNET':
            return "SELECT 1 FROM transactions t WHERE starts_with(t.tx_hash, '0x1_this_is_an_example_query')";
        case 'ETH_TESTNET':
        case 'ETH_MAINNET':
            return "SELECT 1 FROM transactions t WHERE starts_with(t.tx_hash, '0x1_this_is_an_example_query')";
        case 'APTOS_TESTNET':
        case 'APTOS_MAINNET':
            // return `SELECT * FROM transactions`
            return "SELECT 1 FROM transactions t WHERE starts_with(t.hash, '0x1_this_is_an_example_query')";
        default:
            return "SELECT 1 FROM transactions";
    }
}
function getCustomSdkPackage(type) {
    switch (type) {
        case 'SUI_MAINNET':
        case 'SUI_TESTNET':
            return "@mamoru-ai/mamoru-sui-sdk-as";
        case 'BSC_TESTNET':
        case 'BSC_MAINNET':
            return "@mamoru-ai/mamoru-evm-sdk-as";
        case 'ETH_TESTNET':
        case 'ETH_MAINNET':
            return "@mamoru-ai/mamoru-evm-sdk-as";
        case 'APTOS_TESTNET':
        case 'APTOS_MAINNET':
            return "@mamoru-ai/mamoru-aptos-sdk-as";
        default:
            return "@mamoru-ai/mamoru-evm-sdk-as";
    }
}
function getCustomSdkPackageVersion(type) {
    switch (type) {
        case 'SUI_MAINNET':
        case 'SUI_TESTNET':
            return sdk_dependency_versions_1.sdkVersions.sui;
        case 'BSC_TESTNET':
        case 'BSC_MAINNET':
        case 'ETH_TESTNET':
        case 'ETH_MAINNET':
            return sdk_dependency_versions_1.sdkVersions.evm;
        case 'APTOS_TESTNET':
        case 'APTOS_MAINNET':
            return sdk_dependency_versions_1.sdkVersions.aptos;
        default:
            return sdk_dependency_versions_1.sdkVersions.evm;
    }
}
function getCustomSdkCtxName(type) {
    switch (type) {
        case 'SUI_MAINNET':
        case 'SUI_TESTNET':
            return "SuiCtx";
        case 'BSC_TESTNET':
        case 'BSC_MAINNET':
        case 'ETH_TESTNET':
        case 'ETH_MAINNET':
            return "EvmCtx";
        case 'APTOS_TESTNET':
        case 'APTOS_MAINNET':
            return "AptosCtx";
        default:
            return "EvmCtx";
    }
}
function getFilesToCreate(projectPath, ops) {
    var files = __assign({}, constants_1.FILES);
    if (ops.type === 'sql') {
        delete files.WASM_INDEX;
        delete files.WASM_TEST;
    }
    if (ops.type === 'wasm') {
        delete files.QUERIES;
    }
    var fileDirs = {};
    Object.entries(files).forEach(function (_a) {
        var key = _a[0], name = _a[1];
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        fileDirs[key] = path_1.default.join(projectPath, name);
    });
    return fileDirs;
}
function createFile(logger, ops, templatePath, targetPath) {
    var targetFileName = path_1.default.basename(targetPath);
    logger.verbose("Creating \"".concat(targetFileName, "\""));
    var absoluteTemplatePath = path_1.default.join(__dirname, templatePath);
    var templateSrc = fs.readFileSync(absoluteTemplatePath).toString('utf-8');
    var template = handlebars_1.default.compile(templateSrc);
    var result = template(ops);
    fs.writeFileSync(targetPath, result);
}
function createFolder(logger, projectPath, folderPath) {
    var folderName = path_1.default.basename(folderPath);
    logger.verbose("Creating \"".concat(folderName, "\" folder"));
    if (!fs.existsSync(path_1.default.join(projectPath, folderPath))) {
        fs.mkdirSync(path_1.default.join(projectPath, folderName));
    }
}
exports.default = {
    init: init,
    getAugmentedInitOptions: getAugmentedInitOptions,
};
//# sourceMappingURL=init.js.map