#! /usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var colors_1 = __importDefault(require("colors"));
var fs = __importStar(require("fs"));
var commander_1 = require("commander");
var init_1 = __importDefault(require("./commands/init"));
var build_1 = __importDefault(require("./commands/build"));
var publish_1 = __importDefault(require("./commands/publish"));
var spawn_1 = __importDefault(require("./commands/spawn"));
var utils_1 = require("./services/utils");
function parseDirectoryPath(path) {
    if (fs.existsSync(path) && fs.lstatSync(path).isDirectory()) {
        return path;
    }
    else {
        throw new commander_1.InvalidArgumentError('Provided path is not exist or not a folder.');
    }
}
function parseOrSetCurrentDirectoryPath(path) {
    if (!path) {
        return '.';
    }
    if (fs.existsSync(path) && fs.lstatSync(path).isDirectory()) {
        return path;
    }
    else {
        throw new commander_1.InvalidArgumentError('Provided path is not exist or not a folder.');
    }
}
function parseOrCreateDirectoryPath(path) {
    if (path === '') {
        throw new commander_1.InvalidArgumentError('Path is required.');
    }
    if (fs.existsSync(path) && !fs.lstatSync(path).isDirectory()) {
        throw new commander_1.InvalidArgumentError('Provided path is not a folder.');
    }
    if (!fs.existsSync(path)) {
        fs.mkdirSync(path);
        return path;
    }
    if (fs.existsSync(path) && fs.lstatSync(path).isDirectory()) {
        return path;
    }
    throw new commander_1.InvalidArgumentError('Provided path is not exist or not a folder.');
}
function increaseVerbosity(dummyValue, previous) {
    return previous + 1;
}
commander_1.program.option('-v, --verbose', 'define verbosity to show execution logs', increaseVerbosity, 0);
commander_1.program
    .command('init')
    .argument('<path>', 'path to folder with Mamoru project', parseOrCreateDirectoryPath, '')
    .addOption(new commander_1.Option('-t, --type <type>', 'Type of project')
    .choices(['sql', 'wasm'])
    .default('wasm'))
    .addOption(new commander_1.Option('-c, --chain <chain>', 'Chain where the daemon runs')
    .choices((0, utils_1.getAvailableChains)())
    .makeOptionMandatory())
    .addOption(new commander_1.Option('-n, --name <name>', 'Name of the project').default('Default Name'))
    .addOption(new commander_1.Option('-d, --description <description>', 'Description of the project').default('Mamoru Daemon'))
    .addOption(new commander_1.Option('-t, --tags <tags>', 'Tags of the project, comma separated').default('mamoru,daemon'))
    .addOption(new commander_1.Option('-l, --logo <logo>', 'Logo of the project, should be an url').default('https://mamoru.ai/default-daemon-logo.png'))
    .addOption(new commander_1.Option('--subscribable', 'If the project is subscribable, or standalone').default(false))
    .action(function (path, options) {
    init_1.default.init(commander_1.program, path, options);
});
commander_1.program
    .command('build')
    .argument('[path]', 'path to folder with Mamoru project', parseOrSetCurrentDirectoryPath, '.')
    .description('compile project')
    .action(function (path) {
    build_1.default.build(commander_1.program, path);
});
commander_1.program
    .command('publish')
    .argument('[path]', 'path to folder with Mamoru project', parseOrSetCurrentDirectoryPath, '.')
    .option('--rpc <rpcUrl>', 'rpc url of the chain')
    .addOption(new commander_1.Option('--gas <gas>', 'gas fee of the transaction').default('200000'))
    .addOption(new commander_1.Option('-k, --private-key <key>', 'Private key of the account that will be used to publish the project')
    .makeOptionMandatory()
    .env('MAMORU_PRIVATE_KEY'))
    .description('publish project')
    .action(function (path, options) {
    publish_1.default.publish(commander_1.program, path, options);
});
commander_1.program
    .command('launch')
    .description('launch daemon from subscribable metadata')
    .addOption(new commander_1.Option('-m, --metadataid <metadataId>', 'Daemon MetadataId').makeOptionMandatory())
    .option('--rpc <rpcUrl>', 'rpc url of the chain')
    .addOption(new commander_1.Option('--gas <gas>', 'gas fee of the transaction').default('200000'))
    .addOption(new commander_1.Option('-k, --private-key <key>', 'Private key of the account that will be used to publish the project')
    .makeOptionMandatory()
    .env('MAMORU_PRIVATE_KEY'))
    .action(function (options) {
    (0, spawn_1.default)(commander_1.program, {
        metadataId: options.metadataid,
        privateKey: options.privateKey,
        gas: options.gas,
        rpc: options.rpc,
    });
});
commander_1.program.version(
// eslint-disable-next-line @typescript-eslint/no-var-requires
"Mamoru CLI ".concat(require('../package.json').version), '--version');
commander_1.program.configureOutput({
    // Visibly override write routines as example!
    writeOut: function (str) { return process.stdout.write("[OUT] ".concat(str)); },
    writeErr: function (str) { return process.stdout.write("[ERR] ".concat(str)); },
    // Highlight errors in color.
    outputError: function (str, write) {
        return write(colors_1.default.red(str));
    },
});
exports.default = commander_1.program;
//# sourceMappingURL=program.js.map