import { Manifest } from '../types';
import { DirectSecp256k1Wallet } from '@cosmjs/proto-signing';
import { Logger } from './console';
import { MsgRegisterDaemon } from '@mamoru-ai/validation-chain-ts-client/dist/validationchain.validationchain/module';
import { DaemonMetadataContentQuery } from '@mamoru-ai/validation-chain-ts-client/dist/validationchain.validationchain/types/validationchain/validationchain/daemon_metadata_utils';
import { MsgCreateDaemonMetadata, MsgCreateDaemonMetadataResponse, MsgRegisterDaemonResponse, MsgRegisterSnifferResponse } from '@mamoru-ai/validation-chain-ts-client/dist/validationchain.validationchain/types/validationchain/validationchain/tx';
import { Chain_ChainType } from '@mamoru-ai/validation-chain-ts-client/dist/validationchain.validationchain/types/validationchain/validationchain/chain';
import { DaemonMetadata } from '@mamoru-ai/validation-chain-ts-client/src/validationchain.validationchain/types/validationchain/validationchain/daemon_metadata';
import { AxiosInstance } from 'axios';
type TxMsgData = {
    msgResponses: AnyMsg[];
};
type AnyMsg = {
    typeUrl: string;
    value: Uint8Array;
};
export type Msgs = MsgRegisterDaemon | MsgCreateDaemonMetadata | TxMsgData | AnyMsg;
export type ValidationChainMsgs = MsgRegisterDaemon | MsgCreateDaemonMetadata | MsgCreateDaemonMetadataResponse | MsgRegisterDaemonResponse;
declare class ValidationChainService {
    private readonly rpcUrl;
    private readonly privateKey;
    private readonly logger;
    /**
     * ts-client types are broken, so using any for now
     */
    client: any;
    apiClient: AxiosInstance;
    wallet: DirectSecp256k1Wallet;
    apiUrl: string;
    constructor(rpcUrl: string, privateKey: string, logger: Logger);
    private getWallet;
    registerDaemonFromManifest(manifest: Manifest, daemonMetadataId: string): Promise<MsgRegisterDaemonResponse>;
    registerDaemon(daemonMetadataId: string, chainType: Chain_ChainType): Promise<MsgRegisterDaemonResponse>;
    registerSniffer(address: string, chain: Chain_ChainType): Promise<MsgRegisterSnifferResponse>;
    private getVcClient;
    private getApiClient;
    private getTxClient;
    private getQueryClient;
    private getAddress;
    registerDaemonMetadata(manifest: Manifest, queries: DaemonMetadataContentQuery[], wasmModule?: string, gas?: string): Promise<MsgCreateDaemonMetadataResponse>;
    getDaemonMetadataById(id: string): Promise<DaemonMetadata>;
    private getDecoder;
    private decodeProtobuf;
    private decodeTxMessages;
    private formatError;
    private throwOnError;
    /**
     * Get the chain type from the manifest.
     * Exported just for testing purposes
     * @returns
     */
    getChainType(manifest: Manifest): Chain_ChainType;
    /**
     * Utility function that can be used for debug messages from validation-chain protobuf API.
     * Uncomment if you need to debug messages.
     */
    private inspectMsgData;
    private getTxData;
    private getTxDataOnlyResponse;
}
export default ValidationChainService;
//# sourceMappingURL=validation-chain.d.ts.map